 
 /* chname8x v1.0
  * Copyright (C) 2001 Peter Martijn Kuipers <hyper@hysoft-automation.com>
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or 
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY, without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
  * GNU Library General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  * This program changes the name of any program variable contained in an
  * 82p 83p or 8xp file (The program file format for the TI-83, TI-83 and
  * TI-83+).
  * I haven't tested the TI-82 renaming, but it should work
  * 
  */

#include <fcntl.h>
#ifndef __unix__
#include <io.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

int main(int,char**);
unsigned char LL(unsigned short int);
unsigned char HH(unsigned short int);
int strieql(char*,char*);
void helpmsg(void);

int main(int argc,char** argv){
  char* name;
  unsigned char buffer;
  FILE* in;
  FILE* out;
  unsigned short int checksum = 0;
  int i,fpointer = 0,filesize = 0;

  if(argc < 4){
    helpmsg();
    return(1);
  }
  if(strieql(argv[1],argv[2])){
    helpmsg();
    return(1);
  }

  strcpy(name,argv[3]);
  strcat(name,"\0\0\0\0\0\0\0\0");

  if((in = fopen(argv[1],"r")) == NULL){
    puts("Error opening input file\n");
    return(2);
  }

  if((out = fopen(argv[2],"w")) == NULL){
    puts("Error opening output file\n");
    return(2);
  }


  fseek(in,0L,SEEK_END);
  filesize = ftell(in);
  rewind(in);

  printf("filesize: %d\n",filesize);

#ifndef __unix__

  // of unix, i know that the mode is always binary

  setmode(fileno(in), O_BINARY);
  setmode(fileno(out), O_BINARY);

#endif

  //first part doesn't need checksumming

  for(i=0;i<55;i++){
    buffer = fgetc(in);
    fputc(buffer,out);
    fpointer++;
  }

  //now we do need to checksum

  printf("Checksum: %#x %#x %d\n", LL(checksum), HH(checksum),checksum);

  for(i=0;i<5;i++){
    buffer = fgetc(in);
    fputc(buffer,out);
    checksum += buffer;
    fpointer++;
  }

  printf("Checksum: %#x %#x %d\n", LL(checksum), HH(checksum),checksum);

  for(i=0;i<8;i++){
    buffer = fgetc(in); // just to keep the file pointers in sync
    fputc(name[i],out);
    checksum += name[i];
    fpointer++;
  }

  printf("Checksum: %#x %#x %d\n", LL(checksum), HH(checksum),checksum);

  while(fpointer < filesize -2){
    buffer = fgetc(in);
    fputc(buffer,out);
    checksum += buffer;
    fpointer++;
  }

  fputc(LL(checksum),out);
  fputc(HH(checksum),out);

  fclose(out);
  fclose(in);

  printf("Checksum: %#x %#x %d\n",LL(checksum),HH(checksum),checksum);


}

unsigned char LL(unsigned short int toSplit){
  unsigned char LL;
  LL = (toSplit & 0x00FF);
  return(LL);
}

unsigned char HH(unsigned short int toSplit){
  unsigned char HH;
  toSplit >>= 8;
  HH = (toSplit & 0x00FF);
  return(HH);
}

void helpmsg(){
  puts("chname8x v1.0  - changes the internal names of ti-83(+) program files");
  puts("Usage: chname8x source destination newname\n");
  puts("source:      the original file of which you want the name changed");
  puts("destination: the new file containing the renamed program");
  puts("newname:     the new name you want the program to have\n");
  puts("NOTE:");
  puts("1) The source and destination file CAN'T be the same");
  puts("2) I did NOT put any limitations on the characters in the name,");
  puts("   so it's up to you to be careful.\n");
  puts("Have fun with this program");
  puts("-- Peter Martijn Kuipers\n");
  puts("Published under the GNU public license. See the source code for details");
  puts("The complete package can be located at ");
  puts("www.hysoft-automation.com/lib/chname8x.zip for dos, and");
  puts("www.hysoft-automation.com/lib/chname8x.tar.gz for linux");

}  

int strieql(char* str1, char* str2){
  while ((toupper(*str1) == toupper(*str2)) && (*str1)){
    str1++;
    str2++;
  }
  return((*str1 == NULL) && (*str2 == NULL));
}
